@@ -4,13 +4,13 @@
 
 #include <unistd.h> /* STDIN_FILENO */
 #include <stdlib.h> /* getenv */
+#include <string.h> /* strlen/strcpy */
 #include <linux/kd.h>
 #include <linux/keyboard.h>
 
 #include <pobl/bl_debug.h>
 #include <pobl/bl_conf_io.h>
 #include <pobl/bl_mem.h>
-#include <pobl/bl_str.h> /* bl_str_alloca_dup */
 #include <pobl/bl_path.h>
 #include <pobl/bl_dialog.h>
 
@@ -372,8 +372,8 @@ static int process_mouse_event(int source, int action, int64_t time, int x, int
     if (xev.type == ButtonPress) {
       static int click_num;
 
-      /* 800x600 display => (800+600) / 64 = 21 */
-      if (xev.x + xev.y + (_disp.width + _disp.height) / 64 >= _disp.width + _disp.height) {
+      if (xev.x >= _disp.width - _disp.width / 4 &&
+          xev.y >= _disp.height - _disp.height / 4) {
         if (click_num == 0) {
           click_num = 1;
         } else /* if( click_num == 1) */ {
@@ -392,7 +392,14 @@ static int process_mouse_event(int source, int action, int64_t time, int x, int
 
       memset(&prev_xev, 0, sizeof(prev_xev));
     } else if (xev.type == MotionNotify) {
-      if (prev_xev.type == MotionNotify && prev_xev.x == xev.x && prev_xev.y == xev.y) {
+      if (prev_xev.type == MotionNotify &&
+#if 1
+          prev_xev.x - 3 <= xev.x && xev.x <= prev_xev.x + 3 &&
+          prev_xev.y - 3 <= xev.y && xev.y <= prev_xev.y + 3
+#else
+          prev_xev.x == xev.x && prev_xev.y == xev.y
+#endif
+          ) {
         /* Long click on touch panel sends same MotionNotify events continuously. */
 #if 0
         bl_debug_printf("Same event is ignored.\n");
@@ -405,6 +412,7 @@ static int process_mouse_event(int source, int action, int64_t time, int x, int
 
         return 1;
       }
+
       prev_xev = xev;
     }
 
@@ -416,20 +424,21 @@ static int process_mouse_event(int source, int action, int64_t time, int x, int
 #endif
 
     win = get_window(xev.x, xev.y);
-    xev.x -= win->x;
-    xev.y -= win->y;
 
     if (xev.type != ButtonRelease && xev.button == Button1 &&
         /* XXX excludes scrollbar (see x_window_init() called in ui_scrollbar.c) */
         win->width > win->min_width &&
-        win->hmargin < xev.x && xev.x < win->hmargin + win->width &&
-        win->vmargin < xev.y && xev.y < win->vmargin + win->height) {
+        xev.x >= _disp.width - _disp.width / 4 &&
+        xev.y >= _disp.height - _disp.height / 4) {
       /* Start long pressing */
       _display.long_press_counter = 1;
     } else {
       _display.long_press_counter = 0;
     }
 
+    xev.x -= win->x;
+    xev.y -= win->y;
+
     ui_window_receive_event(win, &xev);
   }
 
@@ -694,7 +703,7 @@ void ui_display_idling(ui_display_t *disp /* ignored */
   u_int count;
 
   if (_display.long_press_counter > 0) {
-    if (_display.long_press_counter++ == 10) {
+    if (_display.long_press_counter++ == 20) {
       _display.long_press_counter = 0;
       perform_long_click(_display.app->activity->vm);
     }
@@ -1000,6 +1009,7 @@ void ui_display_send_text_selection(u_char *sel_data, size_t sel_len) {
 }
 
 void ui_display_show_dialog(char *server, char *privkey) {
+  char *server_dup;
   char *user;
   char *host;
   char *port;
@@ -1012,7 +1022,8 @@ void ui_display_show_dialog(char *server, char *privkey) {
   }
 
   if (server == NULL ||
-      !bl_parse_uri(NULL, &user, &host, &port, NULL, &encoding, bl_str_alloca_dup(server))) {
+      !(server_dup = alloca(strlen(server) + 1)) ||
+      !bl_parse_uri(NULL, &user, &host, &port, NULL, &encoding, strcpy(server_dup, server))) {
     user = host = port = encoding = NULL;
   }
 
