@@ -9,7 +9,7 @@ typedef void *bl_dl_handle_t;
 
 bl_dl_handle_t bl_dl_open(const char *dirpath, const char *name);
 
-int bl_dl_close(bl_dl_handle_t handle);
+void bl_dl_close(bl_dl_handle_t handle);
 
 void *bl_dl_func_symbol(bl_dl_handle_t handle, const char *symbol);
 
