@@ -6,21 +6,25 @@
 #include <stdio.h>
 
 #include "bl_types.h" /* size_t */
+#include "bl_def.h" /* HAVE_FGETLN */
 
 typedef struct bl_file {
   FILE* file;
   char *buffer;
+
+#ifndef HAVE_FGETLN
   size_t buf_size;
+#endif
 
 } bl_file_t;
 
 bl_file_t *bl_file_new(FILE* fp);
 
-int bl_file_delete(bl_file_t *file);
+void bl_file_destroy(bl_file_t *file);
 
 bl_file_t *bl_file_open(const char *file_path, const char *mode);
 
-int bl_file_close(bl_file_t *file);
+void bl_file_close(bl_file_t *file);
 
 FILE* bl_fopen_with_mkdir(const char *file_path, const char *mode);
 
