@@ -33,9 +33,8 @@ bl_utmp_t bl_utmp_new(const char *tty, const char *host, int pty_fd) {
   return utmp;
 }
 
-int bl_utmp_delete(bl_utmp_t utmp) {
+void bl_utmp_destroy(bl_utmp_t utmp) {
   removeLineFromUtmp(utmp->tty, utmp->fd);
   free(utmp->tty);
   free(utmp);
-  return 1;
 }
