@@ -7,8 +7,7 @@
 #include <pobl/bl_debug.h>
 
 #include "ef_iso2022_intern.h"
-#include "ef_jisx0208_1983_property.h"
-#include "ef_jisx0213_2000_property.h"
+#include "ef_jis_property.h"
 
 #define IS_C0(c) ((u_char)c <= 0x1f)
 #define IS_C1(c) (0x80 <= (u_char)c && (u_char)c <= 0x9f)
@@ -539,7 +538,7 @@ void ef_iso2022_parser_init_func(ef_iso2022_parser_t *iso2022_parser) {
 
   iso2022_parser->parser.init = iso2022_parser_init;
   iso2022_parser->parser.set_str = ef_iso2022_parser_set_str;
-  iso2022_parser->parser.delete = ef_iso2022_parser_delete;
+  iso2022_parser->parser.destroy = ef_iso2022_parser_destroy;
   iso2022_parser->parser.next_char = ef_iso2022_parser_next_char;
 }
 
@@ -550,7 +549,7 @@ void ef_iso2022_parser_set_str(ef_parser_t *parser, u_char *str, size_t size) {
   parser->is_eos = 0;
 }
 
-void ef_iso2022_parser_delete(ef_parser_t *parser) { free(parser); }
+void ef_iso2022_parser_destroy(ef_parser_t *parser) { free(parser); }
 
 int ef_iso2022_parser_next_char(ef_parser_t *parser, ef_char_t *ch) {
   if (sub_next_char((ef_iso2022_parser_t *)parser, ch) == 0) {
@@ -558,9 +557,9 @@ int ef_iso2022_parser_next_char(ef_parser_t *parser, ef_char_t *ch) {
   }
 
   if (ch->cs == JISX0208_1983) {
-    ch->property = ef_get_jisx0208_1983_property(ch->ch, ch->size);
+    ch->property = ef_get_jisx0208_1983_property(ch->ch);
   } else if (ch->cs == JISX0213_2000_1) {
-    ch->property = ef_get_jisx0213_2000_1_property(ch->ch, ch->size);
+    ch->property = ef_get_jisx0213_2000_1_property(ch->ch);
   } else if (ch->cs == TCVN5712_1_1993) {
     if (0x30 <= ch->ch[0] && ch->ch[0] <= 0x34) {
       ch->property = EF_COMBINING;
