@@ -173,7 +173,7 @@ static void conv_init(ef_conv_t *conv) {
   utf16_conv->is_bof = 1;
 }
 
-static void conv_delete(ef_conv_t *conv) { free(conv); }
+static void conv_destroy(ef_conv_t *conv) { free(conv); }
 
 /* --- global functions --- */
 
@@ -186,7 +186,7 @@ ef_conv_t *ef_utf16_conv_new(void) {
 
   utf16_conv->conv.convert = convert_to_utf16;
   utf16_conv->conv.init = conv_init;
-  utf16_conv->conv.delete = conv_delete;
+  utf16_conv->conv.destroy = conv_destroy;
   utf16_conv->conv.illegal_char = NULL;
 
   utf16_conv->is_bof = 1;
@@ -204,7 +204,7 @@ ef_conv_t *ef_utf16le_conv_new(void) {
 
   utf16_conv->conv.convert = convert_to_utf16le;
   utf16_conv->conv.init = conv_init;
-  utf16_conv->conv.delete = conv_delete;
+  utf16_conv->conv.destroy = conv_destroy;
   utf16_conv->conv.illegal_char = NULL;
 
   utf16_conv->is_bof = 1;
