@@ -491,7 +491,8 @@ static void update_client_side_ui(FcitxClient *client, char *auxup, char *auxdow
       if (ch.property & EF_COMBINING) {
         is_comb = 1;
 
-        if ((*syms->vt_char_combine)(p - 1, ef_char_to_int(&ch), ch.cs, is_fullwidth, is_comb,
+        if ((*syms->vt_char_combine)(p - 1, ef_char_to_int(&ch), ch.cs, is_fullwidth,
+                                     (ch.property & EF_AWIDTH) ? 1 : 0, is_comb,
                                      VT_FG_COLOR, VT_BG_COLOR, 0, 0, LS_UNDERLINE_SINGLE, 0, 0)) {
           continue;
         }
@@ -503,7 +504,8 @@ static void update_client_side_ui(FcitxClient *client, char *auxup, char *auxdow
         is_comb = 0;
       }
 
-      (*syms->vt_char_set)(p, ef_char_to_int(&ch), ch.cs, is_fullwidth, is_comb, VT_FG_COLOR,
+      (*syms->vt_char_set)(p, ef_char_to_int(&ch), ch.cs, is_fullwidth,
+                           (ch.property & EF_AWIDTH) ? 1 : 0, is_comb, VT_FG_COLOR,
                            VT_BG_COLOR, 0, 0, LS_UNDERLINE_SINGLE, 0, 0);
 
       p++;
@@ -688,7 +690,8 @@ static void update_formatted_preedit(FcitxClient *client, GPtrArray *list, int c
         if (ch.property & EF_COMBINING) {
           is_comb = 1;
 
-          if ((*syms->vt_char_combine)(p - 1, ef_char_to_int(&ch), ch.cs, is_fullwidth, is_comb,
+          if ((*syms->vt_char_combine)(p - 1, ef_char_to_int(&ch), ch.cs, is_fullwidth,
+                                       (ch.property & EF_AWIDTH) ? 1 : 0, is_comb,
                                        fg_color, bg_color, 0, 0, LS_UNDERLINE_SINGLE, 0, 0)) {
             continue;
           }
@@ -698,7 +701,8 @@ static void update_formatted_preedit(FcitxClient *client, GPtrArray *list, int c
            */
         }
 
-        (*syms->vt_char_set)(p, ef_char_to_int(&ch), ch.cs, is_fullwidth, is_comb, fg_color,
+        (*syms->vt_char_set)(p, ef_char_to_int(&ch), ch.cs, is_fullwidth,
+                             (ch.property & EF_AWIDTH) ? 1 : 0, is_comb, fg_color,
                              bg_color, 0, 0, LS_UNDERLINE_SINGLE, 0, 0);
 
         p++;
