@@ -197,7 +197,8 @@ static void update_preedit_text(IBusInputContext *context, IBusText *text, gint
       if (ch.property & EF_COMBINING) {
         is_comb = 1;
 
-        if ((*syms->vt_char_combine)(p - 1, ef_char_to_int(&ch), ch.cs, is_fullwidth, is_comb,
+        if ((*syms->vt_char_combine)(p - 1, ef_char_to_int(&ch), ch.cs, is_fullwidth,
+                                     (ch.property & EF_AWIDTH) ? 1 : 0, is_comb,
                                      fg_color, bg_color, 0, 0, LS_UNDERLINE_SINGLE, 0, 0)) {
           continue;
         }
@@ -207,7 +208,8 @@ static void update_preedit_text(IBusInputContext *context, IBusText *text, gint
          */
       }
 
-      (*syms->vt_char_set)(p, ef_char_to_int(&ch), ch.cs, is_fullwidth, is_comb, fg_color,
+      (*syms->vt_char_set)(p, ef_char_to_int(&ch), ch.cs, is_fullwidth,
+                           (ch.property & EF_AWIDTH) ? 1 : 0, is_comb, fg_color,
                            bg_color, 0, 0, LS_UNDERLINE_SINGLE, 0, 0);
 
       p++;
@@ -474,11 +476,9 @@ static void remove_event_source(int complete) {
     ibus_bus_fd = -1;
   }
 
-#ifndef USE_WAYLAND
   if (complete) {
     (*syms->ui_event_source_remove_fd)(IBUS_ID);
   }
-#endif
 }
 
 /*
@@ -700,7 +700,10 @@ static int key_event(ui_im_t *im, u_char key_char, KeySym ksym, XKeyEvent *event
 }
 
 static void set_engine(IBusInputContext *context, gchar *name) {
-  bl_msg_printf("iBus engine is %s\n", name);
+#ifdef DEBUG
+  bl_msg_printf("iBus engine %s -> %s\n",
+                ibus_engine_desc_get_name(ibus_input_context_get_engine(context)), name);
+#endif
   ibus_input_context_set_engine(context, name);
 }
 
@@ -851,11 +854,18 @@ static IBusInputContext *context_new(im_ibus_t *ibus, char *engine) {
     set_engine(context, engine);
   }
 #if (defined(USE_FRAMEBUFFER) || defined(USE_CONSOLE)) && IBUS_CHECK_VERSION(1, 5, 0)
-  else {
+  else /* if (strcmp(ibus_engine_desc_get_name(ibus_input_context_get_engine(context)),
+                     "dummy") == 0) */ {
+    /* Input method engine might not be changed from "dummy" without X server. */
     next_engine(context);
   }
 #endif
 
+#ifdef DEBUG
+  bl_debug_printf("Engine is %s in context_new()\n",
+                  ibus_engine_desc_get_name(ibus_input_context_get_engine(context)));
+#endif
+
   return context;
 }
 
