@@ -280,7 +280,8 @@ static void preedit_update(void *ptr, char *utf8_str, int cursor_offset) {
     if (ch.property & EF_COMBINING) {
       is_comb = 1;
 
-      if ((*syms->vt_char_combine)(p - 1, ef_char_to_int(&ch), ch.cs, is_fullwidth, is_comb,
+      if ((*syms->vt_char_combine)(p - 1, ef_char_to_int(&ch), ch.cs, is_fullwidth,
+                                   (ch.property & EF_AWIDTH) ? 1 : 0, is_comb,
                                    fg_color, bg_color, is_bold, 0, is_underline, 0, 0)) {
         index++;
         continue;
@@ -291,7 +292,8 @@ static void preedit_update(void *ptr, char *utf8_str, int cursor_offset) {
        */
     }
 
-    (*syms->vt_char_set)(p, ef_char_to_int(&ch), ch.cs, is_fullwidth, is_comb, fg_color, bg_color,
+    (*syms->vt_char_set)(p, ef_char_to_int(&ch), ch.cs, is_fullwidth,
+                         (ch.property & EF_AWIDTH) ? 1 : 0, is_comb, fg_color, bg_color,
                          is_bold, 0, is_underline, 0, 0);
 
     p++;
