@@ -644,7 +644,8 @@ static void preedit_pushback(void *ptr, int attr, const char *_str) {
     if (ch.property & EF_COMBINING) {
       is_comb = 1;
 
-      if ((*syms->vt_char_combine)(p - 1, ef_char_to_int(&ch), ch.cs, is_fullwidth, is_comb,
+      if ((*syms->vt_char_combine)(p - 1, ef_char_to_int(&ch), ch.cs, is_fullwidth,
+                                   (ch.property & EF_AWIDTH) ? 1 : 0, is_comb,
                                    fg_color, bg_color, 0, 0, is_underline, 0, 0)) {
         continue;
       }
@@ -654,7 +655,8 @@ static void preedit_pushback(void *ptr, int attr, const char *_str) {
        */
     }
 
-    (*syms->vt_char_set)(p, ef_char_to_int(&ch), ch.cs, is_fullwidth, is_comb, fg_color, bg_color,
+    (*syms->vt_char_set)(p, ef_char_to_int(&ch), ch.cs, is_fullwidth,
+                         (ch.property & EF_AWIDTH) ? 1 : 0, is_comb, fg_color, bg_color,
                          0, 0, is_underline, 0, 0);
 
     p++;
@@ -792,9 +794,12 @@ static void candidate_select(void *p, int index) {
      * (e.g. uim_get_candidate(0) and uim_get_candidate(10) returns the same.)
      */
     if (uim->is_mozc && uim->im.cand_screen->index != index &&
-        uim->im.cand_screen->index / uim->cand_limit != index / uim->cand_limit &&
-        (index % uim->cand_limit) == 0) {
-      candidate_activate(p, uim->im.cand_screen->num_candidates, uim->cand_limit);
+        uim->im.cand_screen->index / uim->cand_limit != index / uim->cand_limit) {
+      int mod = index % uim->cand_limit;
+
+      if (mod == 0 || mod == uim->cand_limit - 1) {
+        candidate_activate(p, uim->im.cand_screen->num_candidates, uim->cand_limit);
+      }
     }
 
     (*uim->im.cand_screen->select)(uim->im.cand_screen, index);
