@@ -377,9 +377,9 @@ static vt_char_t *xterm_get_picture_data(void *p, char *file_path,
 #endif
         {
           /* pua is ambiguous width but always regarded as halfwidth. */
-          vt_char_set(buf_p++, code++, ISO10646_UCS4_1, 0 /* fullwidth */, 0 /* comb */,
-                      VT_FG_COLOR, VT_BG_COLOR, 0 /* bold */, 0 /* italic */, 0 /* line_style */,
-                      0 /* blinking */, 0 /* protected */);
+          vt_char_set(buf_p++, code++, ISO10646_UCS4_1, 0 /* fullwidth */, 0 /* awidth */,
+                      0 /* comb */, VT_FG_COLOR, VT_BG_COLOR, 0 /* bold */, 0 /* italic */,
+                      0 /* line_style */, 0 /* blinking */, 0 /* protected */);
           if ((code & 0x7f) == 0x0) {
 #if 0
             /* for old rlogin */
