@@ -4,7 +4,7 @@
 
 #include <stdio.h>
 #include <ctype.h>
-#include <pobl/bl_str.h>
+#include <pobl/bl_str.h> /* strdup */
 #include <pobl/bl_mem.h> /* free */
 #include <pobl/bl_debug.h>
 #include <glib.h>
