@@ -50,6 +50,7 @@ public:
 class MLView : public BView {
 private:
   int32 buttons;
+  int32 dnd_modifiers;
   const BFont *cur_font;
 
 public:
@@ -72,7 +73,7 @@ private:
 public:
   ConnectDialog();
   bool Go();
-  void ConnectDialog::Position(BWindow *parent);
+  void Position(BWindow *parent);
   virtual void MessageReceived(BMessage *message);
   BTextControl *text;
 };
@@ -395,7 +396,13 @@ void MLView::MouseDown(BPoint where) {
 }
 
 void MLView::MouseMoved(BPoint where, uint32 code, const BMessage *dragMessage) {
-  if (dragMessage || !uiwindow || (IS_UISCREEN(uiwindow) && !((ui_screen_t *)uiwindow)->term)) {
+  if (dragMessage) {
+    Window()->CurrentMessage()->FindInt32((const char*)"modifiers", &dnd_modifiers);
+
+    return;
+  }
+
+  if (!uiwindow || (IS_UISCREEN(uiwindow) && !((ui_screen_t *)uiwindow)->term)) {
     /* It has been already removed from ui_layout or term has been detached. */
     return;
   }
@@ -490,18 +497,41 @@ void MLView::SetFont(const BFont *font, uint32 mask) {
 void MLView::MessageReceived(BMessage *message) {
   if (message->what == B_SIMPLE_DATA) {
     int32 count = 0;
-    entry_ref ref;
 
+#if 0
+    status_t s;
+    int32 action;
+
+    while ((s = message->FindInt32("be:actions", count++, &action)) == B_OK) {
+      bl_debug_printf(BL_DEBUG_TAG "ACTION: %x (COYP %x MOVE %x)\n",
+                      action, B_COPY_TARGET, B_MOVE_TARGET);
+    }
+#ifdef DEBUG
+    if (s == B_NAME_NOT_FOUND) {
+      bl_debug_printf(BL_DEBUG_TAG " be:actions is not found.\n");
+    }
+#endif
+
+    count = 0;
+#endif
+
+    entry_ref ref;
     while (message->FindRef("refs", count++, &ref) == B_OK) {
       BPath path(&ref);
 
-      XSelectionNotifyEvent ev;
-      ev.type = UI_SELECTION_NOTIFIED;
-      ev.data = (char*)path.Path();
-      ev.len = strlen(ev.data);
-
       view_lock(this);
-      ui_window_receive_event(uiwindow, (XEvent*)&ev);
+
+      if (dnd_modifiers & ShiftMask) {
+        (*uiwindow->set_xdnd_config)(uiwindow, NULL, "scp", (char*)path.Path());
+      } else {
+        XSelectionNotifyEvent ev;
+        ev.type = UI_SELECTION_NOTIFIED;
+        ev.data = (char*)path.Path();
+        ev.len = strlen(ev.data);
+
+        ui_window_receive_event(uiwindow, (XEvent*)&ev);
+      }
+
       beos_unlock();
     }
   } else if (message->what == B_INPUT_METHOD_EVENT) {
@@ -730,11 +760,20 @@ void view_draw_string(/* BView */ void *view, ui_font_t *font, ui_color_t *fg_co
   ((BView*)view)->SetHighColor((pixel >> 16) & 0xff, (pixel >> 8) & 0xff, pixel & 0xff,
                                (pixel >> 24) & 0xff);
   ((BView*)view)->SetFont((BFont*)font->xfont->fid);
-  ((BView*)view)->MovePenTo(x, y);
+  /*
+   * https://www.haiku-os.org/legacy-docs/bebook/BView.html
+   * The BeOS draws text one pixel above the logical baseline to maintain
+   * compatibility with an earlier version of one of our most commonly-used font
+   * rasterizers. This affects both fonts and BShapes representing glyphs
+   * (see BFont::GetGlyphShapes(). To draw text at the right place, add one to
+   * the Y coordinate when calling MovePenTo() or specifying a BPoint at which
+   * to begin drawing.
+   */
+  ((BView*)view)->MovePenTo(x, y + 1);
   ((BView*)view)->DrawString(str, len);
 
   if (font->double_draw_gap) {
-    ((BView*)view)->MovePenTo(x + font->double_draw_gap, y);
+    ((BView*)view)->MovePenTo(x + font->double_draw_gap, y + 1);
     ((BView*)view)->DrawString(str, len);
   }
 
@@ -884,7 +923,8 @@ void view_reset_uiwindow(ui_window_t *uiwindow) {
   ((MLView*)uiwindow->my_window)->uiwindow = uiwindow;
 }
 
-void window_alloc(ui_window_t *root, int x, int y, u_int width, u_int height, int popup) {
+void window_alloc(ui_window_t *root, int x, int y, u_int width, u_int height, int popup,
+                  int geom_hint) {
   MLWindow *window;
 
   if (popup) {
@@ -898,16 +938,23 @@ void window_alloc(ui_window_t *root, int x, int y, u_int width, u_int height, in
     view->uiwindow = root;
     window->AddChild(view);
   } else {
-#if 1
     /* XXX */
-    static int count = 0;
-    BRect r = BScreen(window).Frame();
-    int x = ((r.right - width) / 8) * count + 50;
-    int y = ((r.bottom - height) / 8) * count + 50;
-    if (++count == 8) {
-      count = 0;
+    static int x_count = 0;
+    static int y_count = 0;
+    BRect r = BScreen().Frame();
+
+    if ((geom_hint & XValue) == 0) {
+      x = ((r.right - width) / 8) * x_count + 50;
+      if (++x_count == 8) {
+        x_count = 0;
+      }
+    }
+    if ((geom_hint & YValue) == 0) {
+      y = ((r.bottom - height) / 8) * y_count + 50;
+      if (++y_count == 8) {
+        y_count = 0;
+      }
     }
-#endif
 
     window = new MLWindow(BRect(x, y, x + width - 1, y + height - 1),
                           "mlterm", B_DOCUMENT_WINDOW, 0);
@@ -1013,10 +1060,14 @@ void *beos_create_font(const char *font_family, float size, int is_italic, int i
 }
 
 #ifdef USE_OT_LAYOUT
-char *beos_get_font_path(void *bfont) {
-  return NULL;
+char *beos_get_font_family(void *bfont) {
+  font_family family;
+
+  ((BFont*)bfont)->GetFamilyAndStyle(&family, NULL);
+
+  return strdup(family);
 }
-#endif /* USE_OT_LAYOUT */
+#endif
 
 void beos_release_font(void *bfont) {
   delete (BFont*)bfont;
