@@ -15,6 +15,12 @@
 
 static int console_id = -1;
 
+#ifdef KDGKBDIACRUC
+static struct kbdiacrsuc *diacrs_uc;
+#else
+static struct kbdiacrs *diacrs;
+#endif
+
 /* --- static functions --- */
 
 static int get_key_state(void) {
@@ -34,7 +40,69 @@ static int get_key_state(void) {
   }
 }
 
-static int kcode_to_ksym(int kcode, int state) {
+static int get_accented_char(int kb_value, int dead) {
+  u_char table[] = { '`', '\'', '^', '~', '\"', ',' };
+  u_char diacr = table[dead];
+
+  if (dead <= 5) {
+    u_int count;
+
+#ifdef KDGKBDIACRUC
+    if (diacrs_uc == NULL) {
+      if ((diacrs_uc = malloc(sizeof(*diacrs_uc))) == NULL) { /* XXX Leaked */
+        goto end;
+      }
+      ioctl(STDIN_FILENO, KDGKBDIACRUC, diacrs_uc);
+#if 0
+      {
+        u_int count;
+        for (count = 0; count < diacrs_uc.kb_cnt; count++) {
+          bl_debug_printf("UC %x %x %x\n", diacrs_uc.kbdiacruc[count].diacr,
+                          diacrs_uc.kbdiacruc[count].base,
+                          diacrs_uc.kbdiacruc[count].result);
+        }
+      }
+#endif
+    }
+
+    for (count = 0; count < diacrs_uc->kb_cnt; count++) {
+      if (diacrs_uc->kbdiacruc[count].diacr == diacr &&
+          diacrs_uc->kbdiacruc[count].base == kb_value) {
+        return diacrs_uc->kbdiacruc[count].result;
+      }
+    }
+#else
+    if (diacrs == NULL) {
+      if ((diacrs = malloc(sizeof(*diacrs))) == NULL) { /* XXX Leaked */
+        goto end;
+      }
+      ioctl(STDIN_FILENO, KDGKBDIACR, diacrs);
+#if 0
+      {
+        u_int count;
+        for (count = 0; count < diacrs.kb_cnt; count++) {
+          bl_debug_printf("%x %x %x\n", diacrs.kbdiacr[count].diacr,
+                          diacrs.kbdiacr[count].base,
+                          diacrs.kbdiacr[count].result);
+        }
+      }
+#endif
+    }
+
+    for (count = 0; count < diacrs->kb_cnt; count++) {
+      if (diacrs->kbdiacr[count].diacr == diacr &&
+          diacrs->kbdiacr[count].base == kb_value) {
+        return diacrs->kbdiacr[count].result;
+      }
+    }
+#endif
+  }
+
+end:
+  return kb_value;
+}
+
+static int kcode_to_ksym(int kcode, int *state) {
   if (kcode == KEY_ENTER || kcode == KEY_KPENTER) {
     /* KDGKBENT returns '\n'(0x0a) */
     return 0x0d;
@@ -44,23 +112,72 @@ static int kcode_to_ksym(int kcode, int state) {
   } else if (kcode <= KEY_SLASH || kcode == KEY_SPACE || kcode == KEY_YEN || kcode == KEY_RO) {
     struct kbentry ent;
 
-    if (state & ShiftMask) {
-      ent.kb_table = (1 << KG_SHIFT);
+    ent.kb_table = 0;
+    if ((*state) & ShiftMask) {
+      ent.kb_table |= K_SHIFTTAB;
     }
-#ifdef READ_CTRL_KEYMAP
-    else if (state & ControlMask) {
-      ent.kb_table = (1 << KG_CTRL);
-    }
-#endif
-    else {
-      ent.kb_table = 0;
+    if ((*state) & Mod2Mask) {
+      ent.kb_table |= K_ALTTAB;
     }
 
     ent.kb_index = kcode;
 
     if (ioctl(STDIN_FILENO, KDGKBENT, &ent) == 0 && ent.kb_value != K_HOLE &&
         ent.kb_value != K_NOSUCHMAP) {
-      ent.kb_value &= 0xff;
+      static int dead = -1;
+
+#ifdef __DEBUG
+      bl_debug_printf("KDGKBENT idx %x val %x tbl %x\n", ent.kb_index, ent.kb_value, ent.kb_table);
+#endif
+
+      if (ent.kb_value >= 0x1000) {
+        /* See ui_window_get_str() in ui_window.c */
+        ent.kb_value = (0xf000 - (ent.kb_value & 0xf000)) + (ent.kb_value & 0xfff) + 0x1000;
+        dead = -1;
+      } else {
+        if (KTYP(ent.kb_value) == KT_DEAD
+#ifdef KT_DEAD2
+            || KTYP(ent.kb_value) == KT_DEAD2
+#endif
+            ) {
+          dead = ent.kb_value & 0xff;
+
+          return 0;
+        } else {
+          int orig_kb_value = ent.kb_value;
+
+          ent.kb_value &= 0xff;
+
+          if (dead != -1) {
+            ent.kb_value = get_accented_char(ent.kb_value, dead);
+            if (ent.kb_value >= 0x100) {
+              /* See ui_window_get_str() in ui_window.c */
+              ent.kb_value += 0x1000;
+            }
+            dead = -1;
+          }
+
+          if (ent.kb_table & K_ALTTAB) {
+            struct kbentry ent2;
+
+            ent2.kb_table = 0;
+            ent2.kb_index = kcode;
+
+            /*
+             * e.g.) German keyboard
+             *       Alt + ( -> [
+             *       ent2.kb_value = (, orig_kb_value = [
+             */
+            if (ioctl(STDIN_FILENO, KDGKBENT, &ent2) == 0 && orig_kb_value != ent2.kb_value) {
+              *state &= ~ModMask;
+            }
+          }
+        }
+      }
+
+#ifdef __DEBUG
+      bl_debug_printf("-> val %x\n", ent.kb_value);
+#endif
 
 #if 1
       /* XXX linux returns KEY_GRAVE for HankakuZenkaku key. */
@@ -329,6 +446,14 @@ static int open_display(u_int depth) {
     bl_file_set_cloexec(_display.fd);
   }
 
+  /* K_UNICODE is the default value. */
+#if 0
+  {
+    u_long val = K_UNICODE;
+    ioctl(STDIN_FILENO, KDSKBMODE, &val);
+  }
+#endif
+
   _disp.display = &_display;
 
   if ((mouse_num == -1 || (_mouse.fd = open_event_device(mouse_num, NULL)) == -1) &&
@@ -583,7 +708,9 @@ static int receive_key_event(void) {
             }
           } else if (ev.code == KEY_RIGHTCTRL || ev.code == KEY_LEFTCTRL) {
             _display.key_state |= ControlMask;
-          } else if (ev.code == KEY_RIGHTALT || ev.code == KEY_LEFTALT) {
+          } else if (ev.code == KEY_RIGHTALT) {
+            _display.key_state |= (Mod1Mask|Mod2Mask);
+          } else if (ev.code == KEY_LEFTALT) {
             _display.key_state |= Mod1Mask;
           } else if (ev.code == KEY_NUMLOCK) {
             _display.lock_state ^= NLKED;
@@ -591,18 +718,21 @@ static int receive_key_event(void) {
             XKeyEvent xev;
 
             xev.type = KeyPress;
-            xev.ksym = kcode_to_ksym(ev.code, _display.key_state);
-            xev.keycode = ev.code;
             xev.state = _mouse.button_state | _display.key_state;
+            xev.keycode = ev.code;
 
-            receive_event_for_multi_roots(&xev);
+            if ((xev.ksym = kcode_to_ksym(ev.code, &xev.state)) > 0) {
+              receive_event_for_multi_roots(&xev);
+            }
           }
         } else if (ev.value == 0 /* Released */) {
           if (ev.code == KEY_RIGHTSHIFT || ev.code == KEY_LEFTSHIFT) {
             _display.key_state &= ~ShiftMask;
           } else if (ev.code == KEY_RIGHTCTRL || ev.code == KEY_LEFTCTRL) {
             _display.key_state &= ~ControlMask;
-          } else if (ev.code == KEY_RIGHTALT || ev.code == KEY_LEFTALT) {
+          } else if (ev.code == KEY_RIGHTALT) {
+            _display.key_state &= ~(Mod1Mask|Mod2Mask);
+          } else if (ev.code == KEY_LEFTALT) {
             _display.key_state &= ~Mod1Mask;
           }
         }
