@@ -3,7 +3,8 @@
 #ifndef ___UI_H__
 #define ___UI_H__
 
-#ifndef MAC_OS_X_VERSION_MAX_ALLOWED
+/* If MAX_OS_X_VERSION_MAX_ALLOWED is defined, <Cocoa/Cocoa.h> has been already included. */
+#if defined(COCOA_TOUCH) || !defined(MAC_OS_X_VERSION_MAX_ALLOWED)
 
 /* NSText.h */
 
@@ -29,80 +30,78 @@ enum {
 /* NSEvent.h */
 
 enum { /* various types of events */
-       NSLeftMouseDown = 1,
-       NSLeftMouseUp = 2,
-       NSRightMouseDown = 3,
-       NSRightMouseUp = 4,
-       NSMouseMoved = 5,
-       NSLeftMouseDragged = 6,
-       NSRightMouseDragged = 7,
-       NSMouseEntered = 8,
-       NSMouseExited = 9,
-       NSKeyDown = 10,
-       NSKeyUp = 11,
-       NSFlagsChanged = 12,
-       NSAppKitDefined = 13,
-       NSSystemDefined = 14,
-       NSApplicationDefined = 15,
-       NSPeriodic = 16,
-       NSCursorUpdate = 17,
-       NSScrollWheel = 22,
+  NSLeftMouseDown = 1,
+  NSLeftMouseUp = 2,
+  NSRightMouseDown = 3,
+  NSRightMouseUp = 4,
+  NSMouseMoved = 5,
+  NSLeftMouseDragged = 6,
+  NSRightMouseDragged = 7,
+  NSMouseEntered = 8,
+  NSMouseExited = 9,
+  NSKeyDown = 10,
+  NSKeyUp = 11,
+  NSFlagsChanged = 12,
+  NSAppKitDefined = 13,
+  NSSystemDefined = 14,
+  NSApplicationDefined = 15,
+  NSPeriodic = 16,
+  NSCursorUpdate = 17,
+  NSScrollWheel = 22,
 #if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_4
-       NSTabletPoint = 23,
-       NSTabletProximity = 24,
+  NSTabletPoint = 23,
+  NSTabletProximity = 24,
 #endif
-       NSOtherMouseDown = 25,
-       NSOtherMouseUp = 26,
-       NSOtherMouseDragged = 27,
+  NSOtherMouseDown = 25,
+  NSOtherMouseUp = 26,
+  NSOtherMouseDragged = 27,
 #if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_5
-       /* The following event types are available on some hardware on 10.5.2 and
-          later */
-       NSEventTypeGesture = 29,
-       NSEventTypeMagnify = 30,
-       NSEventTypeSwipe = 31,
-       NSEventTypeRotate = 18,
-       NSEventTypeBeginGesture = 19,
-       NSEventTypeEndGesture = 20
+  /* The following event types are available on some hardware on 10.5.2 and later */
+  NSEventTypeGesture = 29,
+  NSEventTypeMagnify = 30,
+  NSEventTypeSwipe = 31,
+  NSEventTypeRotate = 18,
+  NSEventTypeBeginGesture = 19,
+  NSEventTypeEndGesture = 20
 #endif
 };
 
 enum { /* masks for the types of events */
-       NSLeftMouseDownMask = 1 << NSLeftMouseDown,
-       NSLeftMouseUpMask = 1 << NSLeftMouseUp,
-       NSRightMouseDownMask = 1 << NSRightMouseDown,
-       NSRightMouseUpMask = 1 << NSRightMouseUp,
-       NSMouseMovedMask = 1 << NSMouseMoved,
-       NSLeftMouseDraggedMask = 1 << NSLeftMouseDragged,
-       NSRightMouseDraggedMask = 1 << NSRightMouseDragged,
-       NSMouseEnteredMask = 1 << NSMouseEntered,
-       NSMouseExitedMask = 1 << NSMouseExited,
-       NSKeyDownMask = 1 << NSKeyDown,
-       NSKeyUpMask = 1 << NSKeyUp,
-       NSFlagsChangedMask = 1 << NSFlagsChanged,
-       NSAppKitDefinedMask = 1 << NSAppKitDefined,
-       NSSystemDefinedMask = 1 << NSSystemDefined,
-       NSApplicationDefinedMask = 1 << NSApplicationDefined,
-       NSPeriodicMask = 1 << NSPeriodic,
-       NSCursorUpdateMask = 1 << NSCursorUpdate,
-       NSScrollWheelMask = 1 << NSScrollWheel,
+  NSLeftMouseDownMask = 1 << NSLeftMouseDown,
+  NSLeftMouseUpMask = 1 << NSLeftMouseUp,
+  NSRightMouseDownMask = 1 << NSRightMouseDown,
+  NSRightMouseUpMask = 1 << NSRightMouseUp,
+  NSMouseMovedMask = 1 << NSMouseMoved,
+  NSLeftMouseDraggedMask = 1 << NSLeftMouseDragged,
+  NSRightMouseDraggedMask = 1 << NSRightMouseDragged,
+  NSMouseEnteredMask = 1 << NSMouseEntered,
+  NSMouseExitedMask = 1 << NSMouseExited,
+  NSKeyDownMask = 1 << NSKeyDown,
+  NSKeyUpMask = 1 << NSKeyUp,
+  NSFlagsChangedMask = 1 << NSFlagsChanged,
+  NSAppKitDefinedMask = 1 << NSAppKitDefined,
+  NSSystemDefinedMask = 1 << NSSystemDefined,
+  NSApplicationDefinedMask = 1 << NSApplicationDefined,
+  NSPeriodicMask = 1 << NSPeriodic,
+  NSCursorUpdateMask = 1 << NSCursorUpdate,
+  NSScrollWheelMask = 1 << NSScrollWheel,
 #if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_4
-       NSTabletPointMask = 1 << NSTabletPoint,
-       NSTabletProximityMask = 1 << NSTabletProximity,
+  NSTabletPointMask = 1 << NSTabletPoint,
+  NSTabletProximityMask = 1 << NSTabletProximity,
 #endif
-       NSOtherMouseDownMask = 1 << NSOtherMouseDown,
-       NSOtherMouseUpMask = 1 << NSOtherMouseUp,
-       NSOtherMouseDraggedMask = 1 << NSOtherMouseDragged,
+  NSOtherMouseDownMask = 1 << NSOtherMouseDown,
+  NSOtherMouseUpMask = 1 << NSOtherMouseUp,
+  NSOtherMouseDraggedMask = 1 << NSOtherMouseDragged,
 #if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_5
-       /* The following event masks are available on some hardware on 10.5.2 and
-          later */
-       NSEventMaskGesture = 1 << NSEventTypeGesture,
-       NSEventMaskMagnify = 1 << NSEventTypeMagnify,
-       NSEventMaskSwipe = 1U << NSEventTypeSwipe,
-       NSEventMaskRotate = 1 << NSEventTypeRotate,
-       NSEventMaskBeginGesture = 1 << NSEventTypeBeginGesture,
-       NSEventMaskEndGesture = 1 << NSEventTypeEndGesture,
+  /* The following event masks are available on some hardware on 10.5.2 and later */
+  NSEventMaskGesture = 1 << NSEventTypeGesture,
+  NSEventMaskMagnify = 1 << NSEventTypeMagnify,
+  NSEventMaskSwipe = 1U << NSEventTypeSwipe,
+  NSEventMaskRotate = 1 << NSEventTypeRotate,
+  NSEventMaskBeginGesture = 1 << NSEventTypeBeginGesture,
+  NSEventMaskEndGesture = 1 << NSEventTypeEndGesture,
 #endif
-       NSAnyEventMask = NSUIntegerMax
+  NSAnyEventMask = NSUIntegerMax
 };
 
 /* Device-independent bits found in event modifier flags */
@@ -204,27 +203,25 @@ typedef struct { int fd; } Display;
 
 typedef int XIC;
 typedef int XID;
-typedef void* Window; /* NSView */
+typedef void *Window; /* NSView */
 typedef int Drawable;
-typedef struct CGImage* Pixmap;
-typedef struct CGImage* PixmapMask;
+typedef struct CGImage *Pixmap;
+typedef struct CGImage *PixmapMask;
 typedef int GC;
 typedef int Font;
 #ifndef __QUICKDRAWTYPES__
 typedef int Cursor;
 #endif
-typedef int KeyCode; /* Same as type of wparam */
-typedef int KeySym;  /* Same as type of wparam */
+typedef int KeyCode;
+typedef int KeySym;
 
-typedef struct /* Same as definition in X11/X.h */
-    {
+typedef struct /* Same as definition in X11/X.h */ {
   int max_keypermod;
   KeyCode *modifiermap;
 
 } XModifierKeymap;
 
-typedef struct /* Same as definition in X11/X.h */
-    {
+typedef struct /* Same as definition in X11/X.h */ {
   unsigned char byte1;
   unsigned char byte2;
 
@@ -526,5 +523,11 @@ KeySym XStringToKeysym(char *str);
 #undef COMPOSE_DECSP_FONT
 #undef USE_REAL_VERTICAL_FONT
 #undef NO_DISPLAY_FD
+#ifdef COCOA_TOUCH
+#define FLICK_SCROLL
+#else
+#undef FLICK_SCROLL
+#endif
+#define UIWINDOW_SUPPORTS_PREEDITING
 
 #endif
