@@ -15,10 +15,10 @@
 int ui_connect_dialog(char **uri,      /* Should be free'ed by those who call this. */
                       char **pass,     /* Same as uri. If pass is not input, "" is set. */
                       char **exec_cmd, /* Same as uri. If exec_cmd is not input, NULL is set. */
+                      char **privkey,  /* in/out */
                       int *x11_fwd,    /* in/out */
-                      char *display_name, Window parent_window, char **sv_list,
-                      char *def_server /* (<user>@)(<proto>:)<server address>(:<encoding>). */
-                      ) {
+                      char *display_name, Window parent_window,
+                      char *def_server /* (<user>@)(<proto>:)<server address>(:<encoding>). */) {
   char *msg;
 
   if (!(*uri = strdup(def_server))) {
@@ -26,9 +26,12 @@ int ui_connect_dialog(char **uri,      /* Should be free'ed by those who call th
   }
 
   if ((msg = alloca(19 + strlen(*uri) + 1))) {
+    char *p;
+
     sprintf(msg, "Enter password for %s", *uri);
 
-    if ((*pass = cocoa_dialog_password(msg))) {
+    if ((p = cocoa_dialog_password(msg))) {
+      *pass = p;
       *exec_cmd = NULL;
 
       return 1;
@@ -36,6 +39,7 @@ int ui_connect_dialog(char **uri,      /* Should be free'ed by those who call th
   }
 
   free(*uri);
+  *uri = NULL;
 
   return 0;
 }
