@@ -18,10 +18,14 @@
 #include "../ui_selection_encoding.h"
 #include "syswminfo.h"
 
-#ifndef USE_WIN32API
+#if !defined(USE_WIN32API) || defined(USE_LIBSSH2)
 #define MONITOR_PTY
-#include <sys/select.h>
 #include "../vtemu/vt_term_manager.h"
+#ifdef USE_WIN32API
+#include <winsock2.h>
+#else
+#include <sys/select.h>
+#endif
 #endif
 
 #define START_PRESENT_MSEC 4
@@ -335,8 +339,8 @@ static void close_display(ui_display_t *disp) {
 #endif
 
   SDL_DestroyTexture(disp->display->texture);
-  SDL_DestroyWindow(disp->display->window);
   SDL_DestroyRenderer(disp->display->renderer);
+  SDL_DestroyWindow(disp->display->window);
 
   ui_picture_display_closed(disp->display);
 
@@ -347,12 +351,20 @@ static int init_display(Display *display, char *app_name, int x, int y, int hint
   SDL_Rect rect;
 
   if (!display->window) {
+    Uint32 flag;
+
+    if (display->parent) {
+      /* XXX Input Method */
+      flag = SDL_WINDOW_SHOWN | SDL_WINDOW_BORDERLESS | SDL_WINDOW_RESIZABLE |
+             SDL_WINDOW_ALWAYS_ON_TOP | SDL_WINDOW_SKIP_TASKBAR | SDL_WINDOW_UTILITY;
+    } else {
+      flag = SDL_WINDOW_SHOWN | SDL_WINDOW_RESIZABLE | SDL_WINDOW_INPUT_FOCUS;
+    }
+
     if (!(display->window = SDL_CreateWindow(app_name,
                                              (hint & XValue) ? x : SDL_WINDOWPOS_CENTERED,
                                              (hint & YValue) ? y : SDL_WINDOWPOS_CENTERED,
-                                             display->width, display->height,
-                                             SDL_WINDOW_SHOWN | SDL_WINDOW_RESIZABLE |
-                                             SDL_WINDOW_INPUT_FOCUS))) {
+                                             display->width, display->height, flag))) {
       return 0;
     }
 
@@ -462,13 +474,17 @@ static int monitor_ptys(void *p) {
     }
 
     if (maxfd >= 0) {
-      select(maxfd + 1, &read_fds, NULL, NULL, NULL);
+      struct timeval tv;
 
-      SDL_zero(ev);
-      ev.type = pty_event_type;
-      SDL_PushEvent(&ev);
+      tv.tv_usec = 500000; /* 0.5 sec */
+      tv.tv_sec = 0;
+      if (select(maxfd + 1, &read_fds, NULL, NULL, &tv) > 0) {
+        SDL_zero(ev);
+        ev.type = pty_event_type;
+        SDL_PushEvent(&ev);
 
-      SDL_CondWait(pty_cond, mutex);
+        SDL_CondWait(pty_cond, mutex);
+      }
     } else {
       SDL_Delay(100);
     }
