@@ -29,6 +29,12 @@
 #define BUILTIN_SIXEL
 #endif
 
+#ifdef USE_GRF
+#define BPP_PSEUDO 2
+#else
+#define BPP_PSEUDO 1
+#endif
+
 /* --- static functions --- */
 
 static void value_table_refresh(u_char *value_table, /* 256 bytes */
@@ -544,27 +550,18 @@ static int exec_mlimgloader(char *path, u_int width, u_int height, int keep_aspe
 static int exec_mlimgloader(char *path, u_int width, u_int height, int keep_aspect,
                             Pixmap *pixmap) {
   pid_t pid;
-  int fds1[2];
-  int fds2[2];
+  int fds[2];
   ssize_t size;
   u_int32_t tmp;
 
-  if (pipe(fds1) == -1) {
-    return 0;
-  }
-  if (pipe(fds2) == -1) {
-    close(fds1[0]);
-    close(fds1[1]);
-
+  if (pipe(fds) == -1) {
     return 0;
   }
 
   pid = fork();
   if (pid == -1) {
-    close(fds1[0]);
-    close(fds1[1]);
-    close(fds2[0]);
-    close(fds2[0]);
+    close(fds[0]);
+    close(fds[0]);
 
     return 0;
   }
@@ -591,9 +588,8 @@ static int exec_mlimgloader(char *path, u_int width, u_int height, int keep_aspe
       args[6] = NULL;
     }
 
-    close(fds1[1]);
-    close(fds2[0]);
-    if (dup2(fds1[0], STDIN_FILENO) != -1 && dup2(fds2[1], STDOUT_FILENO) != -1) {
+    close(fds[0]);
+    if (dup2(fds[1], STDOUT_FILENO) != -1) {
       execv(args[0], args);
     }
 
@@ -602,20 +598,19 @@ static int exec_mlimgloader(char *path, u_int width, u_int height, int keep_aspe
     exit(1);
   }
 
-  close(fds1[0]);
-  close(fds2[1]);
+  close(fds[1]);
 
   if (!(*pixmap = calloc(1, sizeof(**pixmap)))) {
     goto error;
   }
 
-  if (read(fds2[0], &tmp, sizeof(u_int32_t)) != sizeof(u_int32_t)) {
+  if (read(fds[0], &tmp, sizeof(u_int32_t)) != sizeof(u_int32_t)) {
     goto error;
   }
 
   size = ((*pixmap)->width = tmp) * sizeof(u_int32_t);
 
-  if (read(fds2[0], &tmp, sizeof(u_int32_t)) != sizeof(u_int32_t)) {
+  if (read(fds[0], &tmp, sizeof(u_int32_t)) != sizeof(u_int32_t)) {
     goto error;
   }
 
@@ -628,7 +623,7 @@ static int exec_mlimgloader(char *path, u_int width, u_int height, int keep_aspe
     ssize_t n_rd;
 
     p = (*pixmap)->image;
-    while ((n_rd = read(fds2[0], p, size)) > 0) {
+    while ((n_rd = read(fds[0], p, size)) > 0) {
       p += n_rd;
       size -= n_rd;
     }
@@ -638,8 +633,7 @@ static int exec_mlimgloader(char *path, u_int width, u_int height, int keep_aspe
     }
   }
 
-  close(fds2[0]);
-  close(fds1[1]);
+  close(fds[0]);
 
   /* bl_pty_fork() in vt_pty_unix_new() may block without this in startup. */
 #if 1
@@ -658,8 +652,7 @@ error:
     free(*pixmap);
   }
 
-  close(fds2[0]);
-  close(fds1[1]);
+  close(fds[0]);
 
   return 0;
 }
@@ -710,13 +703,7 @@ static int load_file(Display *display, char *path, u_int width, u_int height, in
         if (ui_picture_modifier_is_normal(pic_mod) /* see modify_pixmap() */) {
           if (((*pixmap)->image = load_sixel_from_data_sharepalette(file_data, &(*pixmap)->width,
                                                                     &(*pixmap)->height)) &&
-              resize_sixel(*pixmap, width, height,
-#ifdef USE_GRF
-                           2
-#else
-                           1
-#endif
-                           )) {
+              resize_sixel(*pixmap, width, height, BPP_PSEUDO)) {
             if (mask) {
               *mask = NULL;
             }
@@ -731,8 +718,9 @@ static int load_file(Display *display, char *path, u_int width, u_int height, in
 
 #ifdef WALL_PICTURE_SIXEL_REPLACES_SYSTEM_PALETTE
     skip_sharepalette:
-      if (!(sixel_cmap = custom_palette) && (sixel_cmap = alloca(sizeof(*sixel_cmap) * 257))) {
-        sixel_cmap[256] = 0; /* No active palette */
+      if (!(sixel_cmap = custom_palette) &&
+          (sixel_cmap = alloca(sizeof(*sixel_cmap) * (SIXEL_PALETTE_SIZE + 1)))) {
+        sixel_cmap[SIXEL_PALETTE_SIZE] = 0; /* No active palette */
         custom_palette = sixel_cmap;
       }
 #endif
@@ -744,7 +732,7 @@ static int load_file(Display *display, char *path, u_int width, u_int height, in
 #ifdef WALL_PICTURE_SIXEL_REPLACES_SYSTEM_PALETTE
         if (sixel_cmap) {
           /* see set_wall_picture() in ui_screen.c */
-          ui_display_set_cmap(sixel_cmap, sixel_cmap[256]);
+          ui_display_set_cmap(sixel_cmap, sixel_cmap[SIXEL_PALETTE_SIZE]);
         }
 #endif
         free(file_data);
@@ -857,12 +845,12 @@ int ui_imagelib_load_file(ui_display_t *disp, char *path, u_int32_t **cardinal,
   return 1;
 }
 
-void ui_delete_image(Display *display, Pixmap pixmap) {
+void ui_destroy_image(Display *display, Pixmap pixmap) {
   free(pixmap->image);
   free(pixmap);
 }
 
-void ui_delete_mask(Display *display, PixmapMask mask /* can be NULL */) {
+void ui_destroy_mask(Display *display, PixmapMask mask /* can be NULL */) {
   if (mask) {
     free(mask);
   }
