@@ -42,7 +42,8 @@
 
 #define INVALID_INDEX (cand_screen->num_candidates)
 
-#ifdef USE_WAYLAND
+#if defined(MANAGE_SUB_WINDOWS_BY_MYSELF) && !defined(MANAGE_ROOT_WINDOWS_BY_MYSELF)
+/* USE_SDL2 || USE_WAYLAND (Display : Window = 1 : 1)  */
 #define DISPLAY(cand_screen) ((cand_screen)->window.disp->display->parent)
 #else
 #define DISPLAY(cand_screen) ((cand_screen)->window.disp)
@@ -88,8 +89,7 @@ static u_int candidate_width(ui_font_manager_t *font_man, ui_im_candidate_t *can
 
     font = ui_get_font(font_man, vt_char_font(&candidate->chars[i]));
 
-    width += ui_calculate_char_width(font, vt_char_code(&candidate->chars[i]),
-                                     vt_char_cs(&candidate->chars[i]), NULL);
+    width += ui_calculate_vtchar_width(font, &candidate->chars[i], NULL);
   }
 
   return width;
@@ -130,7 +130,8 @@ static u_int total_candidate_width(ui_font_manager_t *font_man, ui_im_candidate_
 }
 
 static void adjust_window_position_by_size(ui_im_candidate_screen_t *cand_screen, int *x, int *y) {
-#ifdef USE_WAYLAND
+#if defined(MANAGE_SUB_WINDOWS_BY_MYSELF) && !defined(MANAGE_ROOT_WINDOWS_BY_MYSELF)
+  /* USE_SDL2 || USE_WAYLAND (Display : Window = 1 : 1) */
   if (ACTUAL_WIDTH(&cand_screen->window) > DISPLAY(cand_screen)->width) {
     /* do nothing */
   } else
@@ -144,7 +145,8 @@ static void adjust_window_position_by_size(ui_im_candidate_screen_t *cand_screen
     }
   }
 
-#ifdef USE_WAYLAND
+#if defined(MANAGE_SUB_WINDOWS_BY_MYSELF) && !defined(MANAGE_ROOT_WINDOWS_BY_MYSELF)
+  /* USE_SDL2 || USE_WAYLAND (Display : Window = 1 : 1) */
   if (ACTUAL_HEIGHT(&cand_screen->window) > DISPLAY(cand_screen)->height) {
     /* do nothing */
   } else
@@ -259,6 +261,13 @@ static void draw_screen_vertical(ui_im_candidate_screen_t *cand_screen, u_int to
     win_height = cand_screen->window.height;
   }
 
+  /* XXX */
+#ifdef USE_SDL2
+  if (cand_screen->window.disp->display->resizing) {
+    return;
+  }
+#endif
+
 /*
  * digits and candidates
  */
@@ -306,7 +315,7 @@ static void draw_screen_vertical(ui_im_candidate_screen_t *cand_screen, u_int to
     p = digit_str;
     for (j = 0; j < num_digits + 1; j++) {
       vt_char_init(p);
-      vt_char_set(p++, digit[j], US_ASCII, 0, 0, VT_FG_COLOR, VT_BG_COLOR, 0, 0, 0, 0, 0);
+      vt_char_set(p++, digit[j], US_ASCII, 0, 0, 0, VT_FG_COLOR, VT_BG_COLOR, 0, 0, 0, 0, 0);
     }
 
     draw_str(cand_screen, digit_str, num_digits + 1, 0, i - top, font->height, font->ascent,
@@ -375,7 +384,7 @@ static void draw_screen_vertical(ui_im_candidate_screen_t *cand_screen, u_int to
     p = navi_str;
     for (i = 0; i < len; i++) {
       vt_char_init(p);
-      vt_char_set(p++, navi[i], US_ASCII, 0, 0, VT_FG_COLOR, VT_BG_COLOR, 0, 0, 0, 0, 0);
+      vt_char_set(p++, navi[i], US_ASCII, 0, 0, 0, VT_FG_COLOR, VT_BG_COLOR, 0, 0, 0, 0, 0);
     }
 
     draw_str(cand_screen, navi_str, len, x, cand_screen->num_per_window, font->height,
@@ -435,6 +444,13 @@ static void draw_screen_horizontal(ui_im_candidate_screen_t *cand_screen, u_int
     win_height = cand_screen->window.height;
   }
 
+  /* XXX */
+#ifdef USE_SDL2
+  if (cand_screen->window.disp->display->resizing) {
+    return;
+  }
+#endif
+
   for (i = top; i <= last; i++) {
     int j;
 
@@ -462,7 +478,7 @@ static void draw_screen_horizontal(ui_im_candidate_screen_t *cand_screen, u_int
     p = digit_str;
     for (j = 0; j < num_digits + 1; j++) {
       vt_char_init(p);
-      vt_char_set(p++, digit[j], US_ASCII, 0, 0, VT_FG_COLOR, VT_BG_COLOR, 0, 0, 0, 0, 0);
+      vt_char_set(p++, digit[j], US_ASCII, 0, 0, 0, VT_FG_COLOR, VT_BG_COLOR, 0, 0, 0, 0, 0);
     }
 
     if (draw_index != INVALID_INDEX) {
@@ -649,7 +665,7 @@ static int set_candidate(ui_im_candidate_screen_t *cand_screen, ef_parser_t *par
   u_short info;
 
   info = index >> 16;
-  index &= 0xFF;
+  index &= 0xFFFF;
 
   if (index >= cand_screen->num_candidates) {
 #ifdef DEBUG
@@ -705,6 +721,7 @@ static int set_candidate(ui_im_candidate_screen_t *cand_screen, ef_parser_t *par
 
   while ((*parser->next_char)(parser, &ch)) {
     int is_fullwidth = 0;
+    int is_awidth = 0;
     int is_comb = 0;
 
     if (vt_convert_to_internal_ch(cand_screen->vtparser, &ch) <= 0) {
@@ -715,14 +732,14 @@ static int set_candidate(ui_im_candidate_screen_t *cand_screen, ef_parser_t *par
       is_fullwidth = 1;
     } else if (ch.property & EF_AWIDTH) {
       /* TODO: check col_size_of_width_a */
-      is_fullwidth = 1;
+      is_fullwidth = is_awidth = 1;
     }
 
     if (ch.property & EF_COMBINING) {
       is_comb = 1;
 
-      if (vt_char_combine(p - 1, ef_char_to_int(&ch), ch.cs, is_fullwidth, is_comb, VT_FG_COLOR,
-                          VT_BG_COLOR, 0, 0, 0, 0, 0)) {
+      if (vt_char_combine(p - 1, ef_char_to_int(&ch), ch.cs, is_fullwidth, is_awidth, is_comb,
+                          VT_FG_COLOR, VT_BG_COLOR, 0, 0, 0, 0, 0)) {
         continue;
       }
 
@@ -731,8 +748,8 @@ static int set_candidate(ui_im_candidate_screen_t *cand_screen, ef_parser_t *par
        */
     }
 
-    vt_char_set(p, ef_char_to_int(&ch), ch.cs, is_fullwidth, is_comb, VT_FG_COLOR, VT_BG_COLOR, 0,
-                0, 0, 0, 0);
+    vt_char_set(p, ef_char_to_int(&ch), ch.cs, is_fullwidth, is_awidth, is_comb,
+                VT_FG_COLOR, VT_BG_COLOR, 0, 0, 0, 0, 0);
 
     p++;
     cand_screen->candidates[index].filled_len++;
