@@ -588,6 +588,7 @@ static int is_pcf(const char *file_path) {
 #define HAS_POSITION_INFO_BY_GLYPH(xfont) \
   ((xfont)->glyph_size == (xfont)->glyph_width_bytes * (xfont)->height + 3)
 #define FONT_ROTATED (FONT_ITALIC << 1)
+#define FONTSIZE_IN_FORMAT(format) ((format) & ~(FONT_BOLD | FONT_ITALIC | FONT_ROTATED))
 
 /* --- static variables --- */
 
@@ -702,12 +703,12 @@ static int load_ft(XFontStruct *xfont, const char *file_path, int32_t format, in
 #endif
   }
 
-  fontsize = (format & ~(FONT_BOLD | FONT_ITALIC | FONT_ROTATED));
+  fontsize = FONTSIZE_IN_FORMAT(format);
 
   for (count = 0; count < num_xfonts; count++) {
     if (strcmp(xfonts[count]->file, file_path) == 0 &&
         /* The same face is used for normal, italic and bold. */
-        (xfonts[count]->format & ~(FONT_BOLD | FONT_ITALIC | FONT_ROTATED)) == fontsize) {
+        FONTSIZE_IN_FORMAT(xfonts[count]->format) == fontsize) {
       face = xfonts[count]->face;
 
       goto face_found;
@@ -790,6 +791,15 @@ face_found:
 #endif
         xfont->height = xfont->ascent + descent;
       }
+
+      if (face->glyph->bitmap_top > xfont->ascent) {
+        /* Pragmata Pro Mono: bitmap_top of 'i' and 'j' > xfont->ascent. */
+#ifdef __DEBUG
+        bl_debug_printf("Modify xfont->ascent to %d\n", face->glyph->bitmap_top);
+#endif
+        xfont->height += (face->glyph->bitmap_top - xfont->ascent);
+        xfont->ascent = face->glyph->bitmap_top;
+      }
     }
 
     if (load_glyph(face, format, get_glyph_index(face, 'W'), is_aa)) {
@@ -1034,6 +1044,11 @@ static u_char *get_ft_bitmap_intern(XFontStruct *xfont, u_int32_t code /* glyph
       if (xfont->ascent > face->glyph->bitmap_top) {
         y = xfont->ascent - face->glyph->bitmap_top;
       } else {
+        /*
+         * XXX
+         * If face->glyph->bitmap.rows > xfont->height, 'y = 0' causes unarraynged baseline.
+         * Try to adjust xfont->ascent in load_ft().
+         */
         y = 0;
       }
 
@@ -2316,7 +2331,7 @@ int ui_font_has_ot_layout_table(ui_font_t *font) {
         return 0;
       }
 
-      if (!(font->ot_font = otl_open(font->xfont->face, 0))) {
+      if (!(font->ot_font = otl_open(font->xfont->face))) {
         font->ot_font_not_found = 1;
 
         return 0;
@@ -2335,17 +2350,24 @@ int ui_font_has_ot_layout_table(ui_font_t *font) {
   return 0;
 }
 
-u_int ui_convert_text_to_glyphs(ui_font_t *font, u_int32_t *shaped, u_int shaped_len,
-                                int8_t *offsets, u_int8_t *widths, u_int32_t *cmapped,
-                                u_int32_t *src, u_int src_len, const char *script,
-                                const char *features) {
-  return otl_convert_text_to_glyphs(font->ot_font, shaped, shaped_len, offsets, widths, cmapped,
-                                    src, src_len, script, features, 0);
+u_int ui_convert_text_to_glyphs(ui_font_t *font, u_int32_t *shape_glyphs, u_int num_shape_glyphs,
+                                int8_t *xoffsets, int8_t *yoffsets, u_int8_t *advances,
+                                u_int32_t *noshape_glyphs, u_int32_t *src, u_int src_len,
+                                const char *script, const char *features) {
+#ifdef USE_FREETYPE
+  return otl_convert_text_to_glyphs(font->ot_font, shape_glyphs, num_shape_glyphs,
+                                    xoffsets, yoffsets, advances, noshape_glyphs,
+                                    src, src_len, script, features,
+                                    FONTSIZE_IN_FORMAT(font->xfont->format) *
+                                    (font->size_attr >= DOUBLE_WIDTH ? 2 : 1));
+#else
+  return 0;
+#endif
 }
-
 #endif
 
-u_int ui_calculate_char_width(ui_font_t *font, u_int32_t ch, ef_charset_t cs, int *draw_alone) {
+u_int ui_calculate_char_width(ui_font_t *font, u_int32_t ch, ef_charset_t cs, int is_awidth,
+                              int *draw_alone) {
   if (draw_alone) {
     *draw_alone = 0;
   }
