@@ -7,6 +7,10 @@
 #include <pobl/bl_mem.h>
 #include <pobl/bl_unistd.h> /* bl_usleep */
 #include <mef/ef_ucs4_map.h>
+#ifdef USE_WAYLAND
+#include <mef/ef_utf8_parser.h>
+#endif
+#include <mef/ef_utf16_parser.h>
 
 #include "ui_display.h"
 #include "ui_font.h"
@@ -192,7 +196,7 @@ static void draw_string_intern(ui_window_t *win, XFontStruct *xfont, u_int font_
       if (!output_msg) {
         bl_debug_printf("mlterm is built without USE_BG_TEXTURE, "
                         "so screen is corrupt with -otl option.\n");
-        outupt_msg = 1;
+        output_msg = 1;
       }
     }
 #endif
@@ -1898,7 +1902,7 @@ int ui_window_receive_event(ui_window_t *win, XEvent *event) {
 
 size_t ui_window_get_str(ui_window_t *win, u_char *seq, size_t seq_len, ef_parser_t **parser,
                          KeySym *keysym, XKeyEvent *event) {
-  u_char ch;
+  u_int32_t ch;
 
   if (seq_len == 0) {
     return 0;
@@ -1908,8 +1912,28 @@ size_t ui_window_get_str(ui_window_t *win, u_char *seq, size_t seq_len, ef_parse
 
 #ifdef USE_WAYLAND
   *keysym = event->ksym;
-  if ((ch = ui_display_get_char(event->ksym)) == '\0') {
-    return 0;
+  {
+    char buf[7]; /* UTF_MAX_SIZE + 1 */
+    size_t len = ui_display_get_utf8(buf, event->ksym);
+
+    if (len > 2) {
+      if (seq_len >= len) {
+        static ef_parser_t *utf8_parser;
+
+        if (utf8_parser == NULL) {
+          utf8_parser = ef_utf8_parser_new(); /* XXX leaked */
+        }
+
+        *parser = utf8_parser;
+        memcpy(seq, buf, len);
+      }
+
+      return len;
+    } else if (len == 0) {
+      return 0;
+    } else {
+      ch = buf[0];
+    }
   }
 #else
 #ifdef USE_SDL2
@@ -1934,7 +1958,29 @@ size_t ui_window_get_str(ui_window_t *win, u_char *seq, size_t seq_len, ef_parse
 #endif
 
   if ((*keysym = event->ksym) >= 0x100) {
-    switch (*keysym) {
+#ifndef USE_SDL2
+#if defined(__linux__) || defined(__FreeBSD__)
+    /*
+     * Linux:
+     * 0x00-0xff  : ASCII + ISO8859-1-R
+     * 0x100-0x263: Function Keys (Max: KEY_CLEAR(0x163) + 0x100)
+     * 0x1000-    : Unicode (Use after decrement 0x1000) (See kcode_to_ksym() in ui_display_linux.c)
+     *
+     * FreeBSD:
+     * 0x00-0xff  : ASCII + ISO8859-1-R
+     * 0x100-0x256: Function Keys (Max: KEY_KP0(0x156) + 0x100)
+     * 0x1100-    : Unicode (Use after decrement 0x1000)
+     *              (See receive_key_event() in ui_display_freebsd.c)
+     */
+    if (*keysym >= 0x1000) {
+      ch -= 0x1000;
+
+      goto ucs;
+    } else
+#endif
+#endif
+    {
+      switch (*keysym) {
       case XK_KP_Multiply:
         ch = '*';
         break;
@@ -1954,47 +2000,48 @@ size_t ui_window_get_str(ui_window_t *win, u_char *seq, size_t seq_len, ef_parse
       default:
         if (win->disp->display->lock_state & NLKED) {
           switch (*keysym) {
-            case XK_KP_Insert:
-              ch = '0';
-              break;
-            case XK_KP_End:
-              ch = '1';
-              break;
-            case XK_KP_Down:
-              ch = '2';
-              break;
-            case XK_KP_Next:
-              ch = '3';
-              break;
-            case XK_KP_Left:
-              ch = '4';
-              break;
-            case XK_KP_Begin:
-              ch = '5';
-              break;
-            case XK_KP_Right:
-              ch = '6';
-              break;
-            case XK_KP_Home:
-              ch = '7';
-              break;
-            case XK_KP_Up:
-              ch = '8';
-              break;
-            case XK_KP_Prior:
-              ch = '9';
-              break;
-            case XK_KP_Delete:
-              ch = '.';
-              break;
-            default:
-              return 0;
+          case XK_KP_Insert:
+            ch = '0';
+            break;
+          case XK_KP_End:
+            ch = '1';
+            break;
+          case XK_KP_Down:
+            ch = '2';
+            break;
+          case XK_KP_Next:
+            ch = '3';
+            break;
+          case XK_KP_Left:
+            ch = '4';
+            break;
+          case XK_KP_Begin:
+            ch = '5';
+            break;
+          case XK_KP_Right:
+            ch = '6';
+            break;
+          case XK_KP_Home:
+            ch = '7';
+            break;
+          case XK_KP_Up:
+            ch = '8';
+            break;
+          case XK_KP_Prior:
+            ch = '9';
+            break;
+          case XK_KP_Delete:
+            ch = '.';
+            break;
+          default:
+            return 0;
           }
 
           *keysym = ch;
         } else {
           return 0;
         }
+      }
     }
   } else if (*keysym == XK_Tab && (event->state & ShiftMask)) {
     *keysym = XK_ISO_Left_Tab;
@@ -2009,13 +2056,38 @@ size_t ui_window_get_str(ui_window_t *win, u_char *seq, size_t seq_len, ef_parse
    * Not "<= '_'" but "<= 'z'" because Control + 'a' is not
    * distinguished from Control + 'A'.
    */
-  if ((event->state & ControlMask) && (ch == ' ' || ('@' <= ch && ch <= 'z'))) {
-    seq[0] = (ch & 0x1f);
-  } else {
-    seq[0] = ch;
+  if (ch < 0x80) {
+    if ((event->state & ControlMask) && (ch == ' ' || ('@' <= ch && ch <= 'z'))) {
+      seq[0] = (ch & 0x1f);
+    } else {
+      seq[0] = ch;
+    }
+
+    return 1;
   }
 
-  return 1;
+ucs:
+  if (seq_len >= 2) {
+    static ef_parser_t *utf16_parser;
+
+    if (utf16_parser == NULL) {
+      utf16_parser = ef_utf16_parser_new(); /* XXX leaked */
+    }
+
+    *parser = utf16_parser;
+
+    seq[0] = (ch >> 8) & 0xff;
+    seq[1] = ch & 0xff;
+
+    /*
+     * Disable mod_meta_mode for UTF-16.
+     * e.g.) "mod_meta_mode = 8bit" modifies 0x20AC to 0xA0AC.
+     *       (see key_pressed() in ui_screen.c
+     */
+    event->state &= ~ModMask;
+  }
+
+  return 2;
 }
 
 /*
