@@ -290,9 +290,8 @@ static int set_transparent(ui_window_t *win) {
   /*
    * It is not necessary to set ParentRelative more than once, so
    * this function should be used as follows.
-   * if( ! IS_INHERIT_TRANSPARENT(win) || ! win->wall_picture_is_set)
-   * {
-   *	set_transparent( win) ;
+   * if (!IS_INHERIT_TRANSPARENT(win) || !win->wall_picture_is_set) {
+   *   set_transparent(win);
    * }
    */
 
@@ -769,7 +768,7 @@ static void ui_window_xft_draw_string32(ui_window_t *win, ui_font_t *font, ui_co
     return;
   }
 
-  return (*func)(win, font, fg_color, x, y, str, len);
+  (*func)(win, font, fg_color, x, y, str, len);
 }
 
 static void xft_set_clip(ui_window_t *win, int x, int y, u_int width, u_int height) {
@@ -2748,7 +2747,11 @@ void ui_window_draw_decsp_image_string(ui_window_t *win, ui_font_t *font, ui_col
 #if !defined(NO_DYNAMIC_LOAD_TYPE) || defined(USE_TYPE_XCORE)
 void ui_window_draw_string(ui_window_t *win, ui_font_t *font, ui_color_t *fg_color, int x, int y,
                            u_char *str, u_int len) {
-  /* Removing trailing spaces. */
+  /*
+   * Removing trailing spaces.
+   *
+   * XXX Ignore USE_OT_LAYOUT because OpenTyep layout never works on xcore.
+   */
   while (1) {
     if (len == 0) {
       return;
@@ -2955,6 +2958,7 @@ void ui_window_send_picture_selection(ui_window_t *win, Pixmap pixmap, u_int wid
       image_size = width * height * 4;
       sel_bmp_size = image_size + 54;
 
+      free(sel_bmp);
       if ((sel_bmp = calloc(1, sel_bmp_size))) {
         int x;
         int y;
