@@ -80,6 +80,10 @@ typedef struct vt_pty_mosh {
   size_t buf_len;
   bool ready;
 
+  bool quit_sequence_started;
+  bool lf_entered;
+  /* bool escape_requires_lf; */
+
 } vt_pty_mosh_t;
 
 /* --- static variables --- */
@@ -88,6 +92,11 @@ static vt_pty_mosh_t **ptys;
 static u_int num_ptys;
 static Network::Transport<Network::UserStream, Terminal::Complete> *dead_network;
 
+#ifdef MOSH_SIXEL
+/* for establish_tcp_connection */
+static Network::Transport<Network::UserStream, Terminal::Complete> *cur_network;
+#endif
+
 #ifdef USE_WIN32API
 static int full_width_prop = -1;
 
@@ -99,9 +108,34 @@ static int event_in_pipe;
 static pthread_cond_t event_cond = PTHREAD_COND_INITIALIZER;
 static pthread_mutex_t event_mutex = PTHREAD_MUTEX_INITIALIZER;
 
+static int escape_key = 0x1e;
+static int escape_pass_key = '^';
+static int escape_pass_key2 = '^';
+static std::wstring escape_key_help =
+#ifdef MOSH_SIXEL
+  L"Commands: \".\" quits, \",\" resets, \"^\" gives literal Ctrl-^";
+#else
+  L"Commands: \".\" quits, \"^\" gives literal Ctrl-^";
+#endif
+static bool escape_requires_lf = false;
+
 /* --- static functions --- */
 
+#ifdef MOSH_SIXEL
+void establish_tcp_connection(int port) {
+  if (port <= 0 || cur_network->tcp_sock >= 0) {
+    return;
+  }
+
+  cur_network->tcp_sock = tcp_connect(cur_network->get_remote_addr().sin.sin_addr.s_addr, port);
+  if (cur_network->tcp_sock < 0) {
+    bl_error_printf("Failed to connect to mosh server via tcp to send/recv zmodem packets.\n");
+  }
+}
+#endif
+
 #ifdef USE_WIN32API
+
 int wcwidth(wchar_t ch) {
   ef_property_t prop = ef_get_ucs_property(ch);
 
@@ -124,183 +158,6 @@ int wcwidth(wchar_t ch) {
   }
 }
 
-/*
- * Return value
- * -1: utf8_ch is invalid.
- * -2: utf8_ch is insufficient.
- */
-size_t convert_utf8_to_ucs(u_int32_t *ucs_ch, u_char *utf8_ch, size_t len) {
-  if ((utf8_ch[0] & 0xc0) == 0x80) {
-    return (size_t) -1;
-  } else if ((utf8_ch[0] & 0x80) == 0) {
-    *ucs_ch = utf8_ch[0];
-
-    return 1;
-  } else if ((utf8_ch[0] & 0xe0) == 0xc0) {
-    if (len < 2) {
-      return (size_t) -2;
-    }
-
-    if (utf8_ch[1] < 0x80) {
-      return (size_t) -1;
-    }
-
-    *ucs_ch = (((utf8_ch[0] & 0x1f) << 6) & 0xffffffc0) | (utf8_ch[1] & 0x3f);
-
-    if (*ucs_ch < 0x80) {
-      return (size_t) -1;
-    }
-
-    return 2;
-  } else if ((utf8_ch[0] & 0xf0) == 0xe0) {
-    if (len < 3) {
-      return (size_t) -2;
-    }
-
-    if (utf8_ch[1] < 0x80 || utf8_ch[2] < 0x80) {
-      return (size_t) -1;
-    }
-
-    *ucs_ch = (((utf8_ch[0] & 0x0f) << 12) & 0xffff000) |
-              (((utf8_ch[1] & 0x3f) << 6) & 0xffffffc0) | (utf8_ch[2] & 0x3f);
-
-    if (*ucs_ch < 0x800) {
-      return (size_t) -1;
-    }
-
-    return 3;
-  } else if ((utf8_ch[0] & 0xf8) == 0xf0) {
-    if (len < 4) {
-      return (size_t) -2;
-    }
-
-    if (utf8_ch[1] < 0x80 || utf8_ch[2] < 0x80 || utf8_ch[3] < 0x80) {
-      return (size_t) -1;
-    }
-
-    *ucs_ch = (((utf8_ch[0] & 0x07) << 18) & 0xfffc0000) |
-              (((utf8_ch[1] & 0x3f) << 12) & 0xffff000) |
-              (((utf8_ch[2] & 0x3f) << 6) & 0xffffffc0) | (utf8_ch[3] & 0x3f);
-
-    if (*ucs_ch < 0x10000) {
-      return (size_t) -1;
-    }
-
-    return 4;
-  } else if ((utf8_ch[0] & 0xfc) == 0xf8) {
-    if (len < 5) {
-      return (size_t) -2;
-    }
-
-    if (utf8_ch[1] < 0x80 || utf8_ch[2] < 0x80 || utf8_ch[3] < 0x80 || utf8_ch[4] < 0x80) {
-      return (size_t) -1;
-    }
-
-    *ucs_ch = (((utf8_ch[0] & 0x03) << 24) & 0xff000000) |
-              (((utf8_ch[1] & 0x3f) << 18) & 0xfffc0000) |
-              (((utf8_ch[2] & 0x3f) << 12) & 0xffff000) |
-              (((utf8_ch[3] & 0x3f) << 6) & 0xffffffc0) | (utf8_ch[4] & 0x3f);
-
-    if (*ucs_ch < 0x200000) {
-      return (size_t) -1;
-    }
-
-    return 5;
-  } else if ((utf8_ch[0] & 0xfe) == 0xfc) {
-    if (len < 6) {
-      return (size_t) -2;
-    }
-
-    if (utf8_ch[1] < 0x80 || utf8_ch[2] < 0x80 || utf8_ch[3] < 0x80 || utf8_ch[4] < 0x80 ||
-        utf8_ch[5] < 0x80) {
-      return (size_t) -1;
-    }
-
-    *ucs_ch =
-        (((utf8_ch[0] & 0x01 << 30) & 0xc0000000)) | (((utf8_ch[1] & 0x3f) << 24) & 0xff000000) |
-        (((utf8_ch[2] & 0x3f) << 18) & 0xfffc0000) | (((utf8_ch[3] & 0x3f) << 12) & 0xffff000) |
-        (((utf8_ch[4] & 0x3f) << 6) & 0xffffffc0) | (utf8_ch[4] & 0x3f);
-
-    if (*ucs_ch < 0x4000000) {
-      return (size_t) -1;
-    }
-
-    return 6;
-  } else {
-    return (size_t) -1;
-  }
-}
-
-/*
- * Return value
- * -1: ucs_ch is invalid.
- * -2: utf8_ch is insufficient.
- */
-size_t convert_ucs_to_utf8(u_char *utf8, size_t len, u_int32_t ucs_ch) {
-  if (/* 0x00 <= ucs_ch && */ ucs_ch <= 0x7f) {
-    utf8[0] = ucs_ch;
-
-    return 1;
-  } else if (ucs_ch <= 0x07ff) {
-    if (len < 2) {
-      return (size_t) -2;
-    }
-
-    utf8[0] = ((ucs_ch >> 6) & 0xff) | 0xc0;
-    utf8[1] = (ucs_ch & 0x3f) | 0x80;
-
-    return 2;
-  } else if (ucs_ch <= 0xffff) {
-    if (len < 3) {
-      return (size_t) -2;
-    }
-
-    utf8[0] = ((ucs_ch >> 12) & 0x0f) | 0xe0;
-    utf8[1] = ((ucs_ch >> 6) & 0x3f) | 0x80;
-    utf8[2] = (ucs_ch & 0x3f) | 0x80;
-
-    return 3;
-  } else if (ucs_ch <= 0x1fffff) {
-    if (len < 4) {
-      return (size_t) -2;
-    }
-
-    utf8[0] = ((ucs_ch >> 18) & 0x07) | 0xf0;
-    utf8[1] = ((ucs_ch >> 12) & 0x3f) | 0x80;
-    utf8[2] = ((ucs_ch >> 6) & 0x3f) | 0x80;
-    utf8[3] = (ucs_ch & 0x3f) | 0x80;
-
-    return 4;
-  } else if (ucs_ch <= 0x03ffffff) {
-    if (len < 5) {
-      return (size_t) -2;
-    }
-
-    utf8[0] = ((ucs_ch >> 24) & 0x03) | 0xf8;
-    utf8[1] = ((ucs_ch >> 18) & 0x3f) | 0x80;
-    utf8[2] = ((ucs_ch >> 12) & 0x3f) | 0x80;
-    utf8[4] = ((ucs_ch >> 6) & 0x3f) | 0x80;
-    utf8[5] = (ucs_ch & 0x3f) | 0x80;
-
-    return 5;
-  } else if (ucs_ch <= 0x7fffffff) {
-    if (len < 6) {
-      return (size_t) -2;
-    }
-
-    utf8[0] = ((ucs_ch >> 30) & 0x01) | 0xfc;
-    utf8[1] = ((ucs_ch >> 24) & 0x3f) | 0x80;
-    utf8[2] = ((ucs_ch >> 18) & 0x3f) | 0x80;
-    utf8[3] = ((ucs_ch >> 12) & 0x3f) | 0x80;
-    utf8[4] = ((ucs_ch >> 6) & 0x3f) | 0x80;
-    utf8[5] = (ucs_ch & 0x3f) | 0x80;
-
-    return 6;
-  } else {
-    return (size_t) -1;
-  }
-}
-
 /* Same as vt_pty_ssh.cpp */
 static ssize_t lo_recv_pty(vt_pty_t *pty, u_char *buf, size_t len) {
   return recv(pty->master, (char*)buf, len, 0);
@@ -465,10 +322,25 @@ static int use_loopback(vt_pty_t *pty) {
 }
 
 static int unuse_loopback(vt_pty_t *pty) {
+  char buf[128];
+  ssize_t len;
+
   if (!pty->stored || --(pty->stored->ref_count) > 0) {
     return 0;
   }
 
+  while ((len = (*pty->read)(pty, (u_char*)buf, sizeof(buf))) > 0) {
+    char *p;
+
+    if (!(p = (char*)realloc(((vt_pty_mosh_t *)pty)->buf, ((vt_pty_mosh_t *)pty)->buf_len + len))) {
+      break;
+    }
+
+    memcpy(p + ((vt_pty_mosh_t *)pty)->buf_len, buf, len);
+    ((vt_pty_mosh_t *)pty)->buf = p;
+    ((vt_pty_mosh_t *)pty)->buf_len += len;
+  }
+
 #ifdef USE_WIN32API
   if (pty->read == lo_recv_pty) {
     closesocket(pty->slave);
@@ -503,10 +375,15 @@ static void give_hint_to_overlay(vt_pty_mosh_t *pty_mosh) {
     pty_mosh->network->get_latest_remote_state().state.get_echo_ack());
 }
 
+#ifdef MOSH_SIXEL
+static int zmodem_on_tcp = -2;
+#endif
+
 static void *watch_ptys(void *arg) {
   pthread_detach(pthread_self());
 
   int total_timeout = 0;
+
   while (num_ptys > 0) {
     int timeout = 100;
     int maxfd = 0;
@@ -518,19 +395,35 @@ static void *watch_ptys(void *arg) {
     u_int count;
     u_int prev_num_ptys = num_ptys;
     for (count = 0; count < num_ptys; count++) {
-      std::vector<int> fd_list(ptys[count]->network->fds());
-      for (std::vector<int>::const_iterator it = fd_list.begin();
-           it != fd_list.end(); it++) {
-        FD_SET(*it, &fds);
-        if (*it > maxfd) {
-          maxfd = *it;
+#ifdef MOSH_SIXEL
+      if (zmodem_on_tcp != ptys[count]->network->tcp_sock)
+#endif
+      {
+        std::vector<int> fd_list(ptys[count]->network->fds());
+        for (std::vector<int>::const_iterator it = fd_list.begin();
+             it != fd_list.end(); it++) {
+          FD_SET(*it, &fds);
+          if (*it > maxfd) {
+            maxfd = *it;
+          }
+        }
+
+        int wait = ptys[count]->network->wait_time();
+        if (timeout > wait) {
+          timeout = wait;
         }
       }
 
-      int wait = ptys[count]->network->wait_time();
-      if (timeout > wait) {
-        timeout = wait;
+#ifdef MOSH_SIXEL
+      int fd = ptys[count]->network->tcp_sock;
+      if (fd >= 0 &&
+          (zmodem_on_tcp == -2 || zmodem_on_tcp == fd)) {
+        FD_SET(fd, &fds);
+        if (fd > maxfd) {
+          maxfd = fd;
+        }
       }
+#endif
     }
     pthread_mutex_unlock(&event_mutex);
 
@@ -554,9 +447,52 @@ static void *watch_ptys(void *arg) {
       for (count = 0; count < num_ptys; count++) {
         if (sel_result > 0) {
           std::vector<int> fd_list(ptys[count]->network->fds());
+
+#ifdef MOSH_SIXEL
+          int fd = ptys[count]->network->tcp_sock;
+
+          if (fd >= 0 &&
+              (zmodem_on_tcp == -2 || zmodem_on_tcp == fd)) {
+            if (FD_ISSET(fd, &fds)) {
+              cur_ps = &ptys[count]->network->ps; /* for tcp_recv_from_server() */
+              bool close_socket = !tcp_recv_from_server(fd);
+
+              if (zmodem_processing(&ptys[count]->network->ps)) {
+                zmodem_on_tcp = fd;
+              } else if (zmodem_on_tcp >= 0) {
+                zmodem_on_tcp = -2;
+                close_socket = true;
+              }
+
+              if (close_socket) {
+                closesocket(fd);
+                ptys[count]->network->tcp_sock = -1;
+              }
+
+              ptys[count]->ready = ready = true;
+
+              goto skip_udp;
+            } else if (zmodem_on_tcp == fd) {
+              if (!zmodem_processing(&ptys[count]->network->ps)) {
+                zmodem_on_tcp = -2;
+                closesocket(fd);
+                ptys[count]->network->tcp_sock = -1;
+              }
+
+              goto skip_udp;
+            }
+          }
+#endif
+
           for (std::vector<int>::const_iterator it = fd_list.begin();
                it != fd_list.end(); it++) {
             if (FD_ISSET(*it, &fds)) {
+#ifdef MOSH_SIXEL
+              /* for establish_tcp_connection() which can be called by network->recv() */
+              cur_network = ptys[count]->network;
+              /* for network->recv() */
+              cur_ps = &ptys[count]->network->ps;
+#endif
               ptys[count]->network->recv();
               give_hint_to_overlay(ptys[count]);
               ptys[count]->ready = ready = true;
@@ -564,8 +500,19 @@ static void *watch_ptys(void *arg) {
               break;
             }
           }
+
+#ifdef MOSH_SIXEL
+        skip_udp:
+          ;
+#endif
         }
+#ifdef MOSH_SIXEL
+        pass_seq_change_buf(&ptys[count]->network->ps, 1, false);
+#endif
         ptys[count]->network->tick();
+#ifdef MOSH_SIXEL
+        pass_seq_change_buf(&ptys[count]->network->ps, 0, false);
+#endif
       }
     } else if (dead_network) {
       delete dead_network;
@@ -605,6 +552,11 @@ static void *watch_ptys(void *arg) {
     }
   }
 
+  if (dead_network) {
+    delete dead_network;
+    dead_network = NULL;
+  }
+
 #ifndef USE_WIN32API
   close(event_fds[0]);
   close(event_fds[1]);
@@ -634,11 +586,20 @@ static int final(vt_pty_t *pty) {
   if (dead_network) {
     delete dead_network; /* XXX */
   }
+
   /*
    * If delete pty_mosh->network here, watch_ptys() might fall into infinite loop.
    * (in select()?)
    */
   dead_network = pty_mosh->network;
+#ifdef MOSH_SIXEL
+  if (dead_network >= 0) {
+    closesocket(dead_network->tcp_sock);
+    dead_network->tcp_sock = -1; /* for dead_network->tick() not to send via tcp_sock */
+  }
+#endif
+  dead_network->start_shutdown();
+  dead_network->tick();
 
   delete pty_mosh->overlay;
 
@@ -653,7 +614,14 @@ static int set_winsize(vt_pty_t *pty, u_int cols, u_int rows, u_int width_pix, u
   pthread_mutex_lock(&event_mutex);
 
   pty_mosh->network->get_current_state().push_back(Parser::Resize(cols, rows));
+
+#ifdef MOSH_SIXEL
+  pass_seq_change_buf(&pty_mosh->network->ps, 1, false);
+#endif
   pty_mosh->network->tick();
+#ifdef MOSH_SIXEL
+  pass_seq_change_buf(&pty_mosh->network->ps, 0, false);
+#endif
 
   pty_mosh->overlay->get_prediction_engine().reset();
 
@@ -667,14 +635,129 @@ static ssize_t write_to_pty(vt_pty_t *pty, u_char *buf, size_t len) {
 
   pthread_mutex_lock(&event_mutex);
 
+#ifdef MOSH_SIXEL
+  bool zmodem_cancel = (strcmp((char*)buf, "**\x18\x18\x18\x18\x18\x18\x18\x18"
+                                           "\x08\x08\x08\x08\x08\x08\x08\x08\x08\x08") == 0);
+
+  if (pty_mosh->network->tcp_sock >= 0 &&
+      tcp_send(pty_mosh->network->tcp_sock, (char*)buf, len)) {
+    if (zmodem_cancel) {
+      pass_seq_full_reset(&pty_mosh->network->ps);
+
+      if (zmodem_on_tcp == pty_mosh->network->tcp_sock) {
+        zmodem_on_tcp = -2;
+      }
+      closesocket(pty_mosh->network->tcp_sock);
+      pty_mosh->network->tcp_sock = -1;
+    }
+
+    pass_seq_change_buf(&pty_mosh->network->ps, 1, false);
+    pty_mosh->network->tick();
+    pass_seq_change_buf(&pty_mosh->network->ps, 0, false);
+
+    goto end;
+  } else {
+    if (zmodem_cancel) {
+      pass_seq_full_reset(&pty_mosh->network->ps);
+    }
+  }
+
+  pass_seq_change_buf(&pty_mosh->network->ps, 1, false);
+#endif
+
   pty_mosh->overlay->get_prediction_engine().set_local_frame_sent(
     pty_mosh->network->get_sent_state_last());
 
+#ifdef MOSH_SIXEL
+  /* for overlays.get_prediction_engine().new_user_byte() */
+  cur_ps = &pty_mosh->network->ps;
+#endif
+
   for (size_t count = 0; count < len; count++) {
     pty_mosh->overlay->get_prediction_engine().new_user_byte(buf[count], pty_mosh->framebuffer);
+
+#ifdef MOSH_SIXEL
+    if (pass_seq_has_zmodem(&pty_mosh->network->ps) || zmodem_processing(&pty_mosh->network->ps)) {
+      goto skip;
+    }
+#endif
+
+    if (pty_mosh->quit_sequence_started) {
+      if (buf[count] == '.') {
+        /* Quit sequence is Ctrl-^ . */
+        if (pty_mosh->network->has_remote_addr()) {
+          pty_mosh->overlay->get_notification_engine().set_notification_string(
+            wstring( L"Exiting on user request..." ), true);
+          bl_trigger_sig_child(pty->child_pid);
+        }
+
+        pthread_mutex_unlock(&event_mutex);
+
+        return -1;
+      }
+#ifdef MOSH_SIXEL
+      else if (buf[count] == ',') {
+        /* Reset */
+        pass_seq_full_reset(&pty_mosh->network->ps);
+      }
+#endif
+      else if (buf[count] == 0x1a) {
+        /* Suspend sequence is escape_key Ctrl-Z */
+      } else if ((buf[count] == escape_pass_key) || (buf[count] == escape_pass_key2)) {
+        /* Emulation sequence to type escape_key is escape_key +
+           escape_pass_key (that is escape key without Ctrl) */
+        pty_mosh->network->get_current_state().push_back(Parser::UserByte(escape_key));
+      } else {
+        /* Escape key followed by anything other than . and ^ gets sent literally */
+        pty_mosh->network->get_current_state().push_back(Parser::UserByte(escape_key));
+        pty_mosh->network->get_current_state().push_back(Parser::UserByte(buf[count]));
+      }
+
+      pty_mosh->quit_sequence_started = false;
+
+      if (pty_mosh->overlay->get_notification_engine().get_notification_string() ==
+          escape_key_help) {
+        pty_mosh->overlay->get_notification_engine().set_notification_string(L"");
+      }
+
+      continue;
+    }
+
+    pty_mosh->quit_sequence_started = (escape_key > 0) && (buf[count] == escape_key) &&
+                                      (pty_mosh->lf_entered || !escape_requires_lf) ;
+
+    if (pty_mosh->quit_sequence_started) {
+      pty_mosh->lf_entered = false;
+      pty_mosh->overlay->get_notification_engine().set_notification_string(escape_key_help,
+                                                                           true, false);
+
+      continue;
+    }
+
+    /* LineFeed, Ctrl-J, '\n' or CarriageReturn, Ctrl-M, '\r' */
+    pty_mosh->lf_entered = (buf[count] == 0x0A || buf[count] == 0x0D);
+
+    /* XXX */
+#if 0
+    if (buf[count] == 0x0C) { /* Ctrl-L */
+      /* Repaint is requested */
+    }
+#endif
+
+  skip:
     pty_mosh->network->get_current_state().push_back(Parser::UserByte(buf[count]));
   }
-  pty_mosh->network->tick();
+
+#ifdef MOSH_SIXEL
+  pass_seq_change_buf(&pty_mosh->network->ps, 0, true /* clear pass seq generated by overlay */);
+  pass_seq_change_buf(&pty_mosh->network->ps, 1, false);
+#endif
+  pty_mosh->network->tick(); /* send UserByte(buf) above to the server */
+#ifdef MOSH_SIXEL
+  pass_seq_change_buf(&pty_mosh->network->ps, 0, false);
+
+end:
+#endif
 
   pthread_mutex_unlock(&event_mutex);
 
@@ -703,7 +786,7 @@ static ssize_t read_pty(vt_pty_t *pty, u_char *buf, size_t len) {
   } else if (((int64_t)pty_mosh->network->get_remote_state_num()) < 0) {
     bl_trigger_sig_child(pty->child_pid);
 
-    return -1;
+    return 0;
   }
 
 #if 0
@@ -715,14 +798,21 @@ static ssize_t read_pty(vt_pty_t *pty, u_char *buf, size_t len) {
                   pty_mosh->network->get_sent_state_acked_timestamp());
 #endif
 
-  size_t prev_len;
-  if ((prev_len = min(pty_mosh->buf_len, len)) > 0) {
+  size_t prev_len = min(pty_mosh->buf_len, len);
+
+  if (pty_mosh->buf_len > 0) {
+    /*
+     * Even if pty_mosh->buf_len > 0, prev_len may be 0 because len may be 0.
+     * In this case, it is necessary to enter this block not to change
+     * pty_mosh->buf_len.
+     */
     memcpy(buf, pty_mosh->buf, prev_len);
     buf += prev_len;
     len -= prev_len;
 
     if ((pty_mosh->buf_len -= prev_len) == 0) {
       free(pty_mosh->buf);
+      pty_mosh->buf = NULL;
     } else {
       memmove(pty_mosh->buf, pty_mosh->buf + prev_len, pty_mosh->buf_len);
 
@@ -755,6 +845,51 @@ static ssize_t read_pty(vt_pty_t *pty, u_char *buf, size_t len) {
     }
   }
 
+#ifdef MOSH_SIXEL /* defined in parser.h */
+  pthread_mutex_lock(&event_mutex);
+
+  size_t seq_len;
+  char *seq = pass_seq_get(&pty_mosh->network->ps, &seq_len);
+
+  if (seq) {
+    /* XXX in case len < 8 */
+    if (len >= 8) {
+      size_t prepend;
+
+      if (!pass_seq_has_zmodem(&pty_mosh->network->ps) && memcmp(seq, "\x1bP", 2) == 0) {
+        memcpy(buf, "\x1b[?8800h", 8); /* for DRCS-Sixel */
+        len -= 8;
+        memcpy(buf + 8, seq, min(seq_len, len));
+        prepend = 8;
+      } else {
+        memcpy(buf, seq, min(seq_len, len));
+        prepend = 0;
+      }
+
+      if (seq_len > len) {
+        if ((pty_mosh->buf = (char*)malloc(seq_len - len))) {
+          pty_mosh->buf_len = seq_len - len;
+          memcpy(pty_mosh->buf, seq + len, pty_mosh->buf_len);
+        }
+      } else {
+        len = seq_len;
+      }
+
+      pass_seq_reset(&pty_mosh->network->ps);
+
+      pthread_mutex_unlock(&event_mutex);
+
+      return len + prepend + prev_len;
+    }
+  } else if (zmodem_processing(&pty_mosh->network->ps)) {
+    pthread_mutex_unlock(&event_mutex);
+
+    return prev_len;
+  }
+
+  pthread_mutex_unlock(&event_mutex);
+#endif
+
   pthread_mutex_lock(&event_mutex);
   Terminal::Framebuffer framebuffer = pty_mosh->network->get_latest_remote_state().state.get_fb();
   pty_mosh->overlay->apply(framebuffer);
@@ -820,9 +955,12 @@ vt_pty_t *vt_pty_mosh_new(const char *cmd_path, /* If NULL, child prcess is not
 
   if (strcmp(host, ip) != 0) {
     char *new_uri;
+
     if ((new_uri = (char*)alloca(strlen(uri) - strlen(host) + strlen(ip) + 1))) {
-      strcpy(new_uri, uri);
-      uri = bl_str_replace(new_uri, host, ip);
+      const char *p = strstr(uri, host);
+      memcpy(new_uri, uri, p - uri);
+      strcpy(new_uri + (p - uri), ip);
+      uri = strcat(new_uri, uri + (p - uri + strlen(host)));
     }
   }
 
@@ -899,6 +1037,12 @@ vt_pty_t *vt_pty_mosh_new(const char *cmd_path, /* If NULL, child prcess is not
     while (cmd_argv[argv_count++]);
   }
 
+#ifdef MOSH_SIXEL
+  if (getenv("MOSH_NO_TCP")) {
+    argv_count++;
+  }
+#endif
+
   u_int env_count = 0;
   if (env) {
     while (env[env_count++]);
@@ -906,10 +1050,16 @@ vt_pty_t *vt_pty_mosh_new(const char *cmd_path, /* If NULL, child prcess is not
 
   char **argv;
   if (argv_count + env_count > 0 &&
-      (argv = (char**)alloca(sizeof(base_argv) + sizeof(char*) * (argv_count + env_count * 2 + 8)))) {
+      (argv = (char**)alloca(sizeof(base_argv) + sizeof(char*) * (argv_count + env_count * 2)))) {
     memcpy(argv, base_argv, sizeof(base_argv));
     char **argv_p = argv + sizeof(base_argv) / sizeof(base_argv[0]) - 1;
 
+#ifdef MOSH_SIXEL
+    if (getenv("MOSH_NO_TCP")) {
+      *(argv_p++) = "-t";
+    }
+#endif
+
     for (env_count = 0; env[env_count]; env_count++) {
       /* "COLORFGBG=default;default" breaks following environmental variables. */
       if (!strchr(env[env_count], ';')) {
@@ -1041,6 +1191,8 @@ vt_pty_t *vt_pty_mosh_new(const char *cmd_path, /* If NULL, child prcess is not
 #ifdef TEST_BY_SSH
   close(fds[0]);
 #else
+  /* See vt_pty_destroy() in vt_pty.c */
+  free(ssh->cmd_line);
   (*ssh->final)(ssh);
   free(ssh);
 #endif
